#!/usr/bin/env python
"""
Command line utility to import segy data files into an SQLite database
file.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segysql.py,v 1.12 2005/06/19 03:56:41 schwehr Exp $

http://www.python.org/doc/lib/module-optparse.html

http://schwehr.org/software/segy-py/
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.12 $' # See man ident
__date__ = '$Date: 2005/06/19 03:56:41 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """Lots of people for helping me, especially Neal
Driscoll and Lisa Tauxe."""

from segy import Segy
import segy
import sqlite
import os
import sys

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.12 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-c','--create',dest='create',action="store_true",default=True,
		    help='create the tables for segyFile and segyTrace')
myparser.add_option('-C','--no-create',dest='create',action='store_false',
		    help='do NOT create the tables.  Assume that they have already been created')
myparser.add_option('-d','--driver',dest="drv",
                  help="specify which driver to parse data file.  [default: %default]",
                  default="segy_drv_rev1")
myparser.add_option('-D','--delete',dest='delete',default=False,action='store_true',
		    help='delete previous db if it exists  [default: %default]')
myparser.add_option('-f','--file',dest='dbfilename',default='segy.db',
		    help='file for the sqlite database  [default: %default]')
myparser.add_option('-s','--shortList',dest='short', default=False, action='store_true',
		    help='use short list for keys to use [default: %default]')
myparser.add_option('-t','--test',dest='testing',default=False, action='store_true',
		    help='only put the first few traces of each segy file in the database')

if __name__ == "__main__":
        (options,args) = myparser.parse_args()
	dbName = options.dbfilename
	if options.delete and os.access(dbName,os.W_OK):
            os.remove(dbName)
	cx = sqlite.connect(dbName)
	cu = cx.cursor()

	#
	# CREATE the database tables
	#
	if options.create:
		if options.short:
			drv = segy.getDriverModule(options.drv)

			cu.execute(segy.sqlCreateFileTable(drivername=options.drv,specificKeys=drv.fileHeaderShortList))
			cu.execute(segy.sqlCreateTraceTable(drivername=options.drv,specificKeys=drv.traceHeaderShortList))
		else:
			cu.execute(segy.sqlCreateFileTable(drivername=options.drv))
			cu.execute(segy.sqlCreateTraceTable(drivername=options.drv))
		cx.commit()

	#
	# POPULATE the tables
	#
	#[ 'bpsio01L00.xstar',  'bpsio04L03.xstar']:
	for filename in args:
		xstar = Segy(filename,drivername=options.drv)
		if options.short:
			cu.execute(xstar.header.sqlInsert(filename=filename,specificKeys=xstar.drv.fileHeaderShortList))
		else:
			cu.execute(xstar.header.sqlInsert(filename=filename))
		cx.commit()

		cu.execute('SELECT fileKey FROM segyFile WHERE filename=\''+filename+'\'')
		fileKey = cu.fetchone()['fileKey']
		print 'fileKey = ',fileKey
		
		for i in range(1,xstar.getNumberOfTraces()+1):
			t = xstar.getTraceHdr(i)
			#if options.testing and (i > 1000): break  # FIX: remove this hack

			if 0==i%500:
				traceNo = t.getHeader('TraceNo')
				x = t.getHeader('X')/3600.
				y = t.getHeader('Y')/3600.
				print i,':   ',traceNo, x,y
			if (options.short):
				cu.execute(t.sqlInsert(specificKeys=xstar.drv.traceHeaderShortList,traceNumber=i,fileKey=fileKey))
			else:
				cu.execute(t.sqlInsert(traceNumber=i,fileKey=fileKey))
		cx.commit()	# Flush this to the db!

	
